/*****************************************************************************
*       The Graphics Engine version 1.20                                     *
*                                                                            *
*       The Graphics Engine code and documentation are Copyright (c) 1993    *
*	by Matthew Hildebrand.                                               *
*                                                                            *
*       Unauthorised usage or modification of any or all of The Graphics     *
*       Engine is strictly prohibited.                                       *
*****************************************************************************/

#include <alloc.h>
#include <conio.h>
#include <dir.h>
#include <dos.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "tgemouse.h"
#include "tgefont.h"
#include "tge.h"


#ifndef __TURBOC__
  #define random(num)		(rand() % num)
  #define randomize()     	srand((unsigned)time(NULL))
#endif

#define ESC	27


void setup(int argc, char *argv[]);
void lineDemo(void);
void putPixelDemo(void);
void drawRectDemo(void);
void filledRectDemo(void);
void ellipseDemo(void);
void filledEllipseDemo(void);
void circleDemo(void);
void filledCircleDemo(void);
void viewportDemo(void);
void setupViewport(void);
void viewportPutImageDemo(int delayTime);
void viewportRandomDemo();
void paletteDemo(void);
void mouseDemo(void);
void cycleColour(int *colour, int *increment);
void forceExtension(char *filename, char *extension);
void signOff(void);


unsigned char tgeLogo[] = {
  24,0,9,0,
  0,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,0,1,1,1,1,1,1,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,1,1,0,0,0,0,1,1,0,0,0,0,0,0,1,1,0,0,0,0,0,
  0,0,0,1,1,0,0,0,0,1,1,0,0,0,0,0,0,1,1,0,0,0,0,0,
  0,0,1,1,0,0,0,0,1,1,0,0,1,1,1,0,1,1,1,1,1,0,0,0,
  0,0,1,1,0,0,0,0,1,1,0,0,0,1,1,0,1,1,1,1,0,0,0,0,
  0,1,1,0,0,0,0,1,1,0,0,0,0,1,0,1,1,0,0,0,0,0,0,0,
  0,1,1,0,0,0,0,1,1,1,1,1,1,1,0,1,1,1,1,1,1,0,0,0,
  1,1,0,0,0,0,1,1,1,1,1,1,1,0,1,1,1,1,1,1,0,0,0,0
};
#define LOGOWIDE	((unsigned*)tgeLogo)[0]
#define LOGODEEP	((unsigned*)tgeLogo)[1]


Font font("8x16.fnt");
int count;
int colour;
int maxx, maxy;
unsigned maxColour;
void far *bground;


//*****
//***** Program entry
//*****

void main(int argc, char *argv[])
{
  setup(argc, argv);

  //*** run the demos
  lineDemo();
  putPixelDemo();
  drawRectDemo();
  filledRectDemo();
  ellipseDemo();
  filledEllipseDemo();
  circleDemo();
  filledCircleDemo();
  viewportDemo();
  paletteDemo();
  mouseDemo();

  deInitGraphics();			// restore text and quit
}


//*****
//***** Set up at program start
//*****

void setup(int argc, char *argv[])
{
  char filename[80];

  if (argc < 2)				// Parse command line
  {
    printf("TGEDEMO 1.3  Copyright (c) 1993 by Matthew Hildebrand\n\n"
	   "    Usage:  TGEDEMO driver\n\n");
    exit(1);
  }

  if (!font.status())			// Ensure font loaded correctly
  {
    printf("Error loading 8X16.FNT\n\n");
    exit(1);
  }

  strcpy(filename, argv[1]);
  forceExtension(filename, ".DRV");	// Load the specified driver
  if (loadGraphDriver(filename) != TGE_SUCCESS)
  {
    printf("Error loading \"%s\".\n\n", filename);
    exit(1);
  }

  maxx = MAXX;                           // set up variables
  maxy = MAXY;
  maxColour = MAXCOLOUR;

  bground = farmalloc(imageSize(0,0,23,8));

  if (!initGraphics())			// initialize graphics mode
  {
    printf("Unable to initialize graphics hardware.\n\n");
    exit(1);
  }

  randomize();
  font.foreground(colourCloseToX(0,0,200,0));	// set font foreground colour

  atexit(signOff);			// display sign-off screen at exit
  atexit(unloadGraphDriver);		// unload graphics driver at exit
}


//*****
//***** line() demo
//*****

void lineDemo(void)
{
  static char str[] = "TGE supports many functions.";
  int depth = font.deep(str);

  font.put((maxx+1)/2-font.wide(str)/2, maxy-depth, str);

  for (; !kbhit();)
    line(random(maxx+1), random(maxy-depth), random(maxx+1),
    	random(maxy-depth), random(maxColour));

  if (getch() == ESC)
  {
    deInitGraphics();
    exit(0);
  }

  colour = random(maxColour);
  for (count=maxx; count>=0; count-=2)
  {
    line(0, 0, maxx-count, maxy, colour);
    line(maxx, maxy, count, 0, colour);
  }
  clearGraphics(random(maxColour));
}


//*****
//***** putPixel() demo
//*****

void putPixelDemo(void)
{
  for (; !kbhit();)
  {
    for (count=0; count<2000; count++)
      putPixel(random(maxx+1), random(maxy+1), random(maxColour));
  }

  if (getch() == ESC)
  {
    deInitGraphics();
    exit(0);
  }

  clearGraphics(random(maxColour));
}


//*****
//***** drawRect() demo
//*****

void drawRectDemo(void)
{
  while (!kbhit())
    drawRect(random(maxx+1), random(maxy+1), random(maxx+1),
    		random(maxy+1), random(maxColour));

  if (getch() == ESC)
  {
    deInitGraphics();
    exit(0);
  }

  clearGraphics(random(maxColour));
}


//*****
//***** filledRect() demo
//*****

void filledRectDemo(void)
{
  while (!kbhit())
    filledRect(random(maxx+1), random(maxy+1), random(maxx+1),
    		random(maxy+1), random(maxColour));

  if (getch() == ESC)
  {
    deInitGraphics();
    exit(0);
  }

  clearGraphics(random(maxColour));
}


//*****
//***** ellipse() demo
//*****

void ellipseDemo(void)
{
  while (!kbhit())
    ellipse(random(maxx+1), random(maxy+1), random(maxx/8)*2+10,
    		random(maxy/8)*2+10, random(maxColour));

  if (getch() == ESC)
  {
    deInitGraphics();
    exit(0);
  }

  clearGraphics(random(maxColour));
}


//*****
//***** filledEllipse() demo
//*****

void filledEllipseDemo(void)
{
  while (!kbhit())
    filledEllipse(random(maxx+1), random(maxy+1), random(maxx/8)*2+10,
    	random(maxy/8)*2+10, random(maxColour));

  if (getch() == ESC)
  {
    deInitGraphics();
    exit(0);
  }

  clearGraphics(random(maxColour));
}


//*****
//***** circle() demo
//*****

void circleDemo(void)
{
  while (!kbhit())
    circle(random(maxx+1), random(maxy+1), random(maxx/8)+10,
    		random(maxColour));

  if (getch() == ESC)
  {
    deInitGraphics();
    exit(0);
  }

  clearGraphics(random(maxColour));
}


//*****
//***** filledCircle() demo
//*****

void filledCircleDemo(void)
{
  while (!kbhit())
    filledCircle(random(maxx+1), random(maxy+1), random(maxx/8)+10,
    		random(maxColour));

  if (getch() == ESC)
  {
    deInitGraphics();
    exit(0);
  }

  clearGraphics(0);
}


//*****
//***** Viewport demo
//*****

void viewportDemo(void)
{
  int x1, y1, x2, y2;
  static char str1[] = "Viewports can be very useful.";
  static char str2[] = "(Now that it's slower, the";
  static char str3[] = "image's features can be seen.)";
  static char str4[] = "Many functions work with viewports,";
  static char str5[] = "and at considerable speeds.";

  x1 = 2 * (maxx+1)/5;
  y1 = 2 * (maxy+1)/5;
  x2 = maxx - x1;
  y2 = maxy - y1;

  setupViewport();
  font.put((maxx+1)/2-font.wide(str1)/2, (maxy+1)/10, str1);
  setViewport(x1, y1, x2, y2);
  viewportPutImageDemo(0);

  setupViewport();
  font.put((maxx+1)/2-font.wide(str2)/2, (maxy+1)/10, str2);
  font.put((maxx+1)/2-font.wide(str3)/2, (maxy+1)/10+font.deep(str1),
  	str3);
  setViewport(x1, y1, x2, y2);
  viewportPutImageDemo(12);
  setViewport(0, 0, maxx, maxy);

  x1 = (maxx+1)/5;			// make a larger viewport
  y1 = (maxy+1)/5;
  x2 = maxx - x1;
  y2 = maxy - y1;
  clearGraphics(colourCloseTo(90,90,90));
  filledRect(x1, y1, x2, y2, colourCloseTo(0,0,0));
  font.put((maxx+1)/2-font.wide(str4)/2, (maxy+1)/30, str4);
  font.put((maxx+1)/2-font.wide(str5)/2, (maxy+1)/30+font.deep(str4), str5);
  setViewport(x1, y1, x2, y2);
  viewportRandomDemo();

  setViewport(0, 0, maxx, maxy);		// restore the viewport
  clearGraphics(random(maxColour));
}


//*****
//***** Set up the viewport
//*****

void setupViewport(void)
{
  int x1, y1, x2, y2;

  x1 = 2 * (maxx+1)/5;
  y1 = 2 * (maxy+1)/5;
  x2 = maxx - x1;
  y2 = maxy - y1;

  clearGraphics(colourCloseTo(90,90,90));
  filledRect(x1, y1, x2, y2, colourCloseTo(0,0,0));
  setViewport(x1, y1, x2, y2);                  // set viewport

  // center
  putImage((x2+x1+1)/2-LOGOWIDE/2, (y2+y1+1)/2-LOGODEEP/2, tgeLogo);
  // upper left
  putImage(x1-LOGOWIDE/2, y1-LOGODEEP/2, tgeLogo);
  // upper middle
  putImage(((x2+x1+1)/2)-LOGOWIDE/2, y1-LOGODEEP/2, tgeLogo);
  // upper right
  putImage(x2-LOGOWIDE/2, y1-LOGODEEP/2, tgeLogo);
  // middle right
  putImage(x2-LOGOWIDE/2, (y2+y1+1)/2-LOGODEEP/2, tgeLogo);
  // lower right
  putImage(x2-LOGOWIDE/2, y2-LOGODEEP/2, tgeLogo);
  // lower middle
  putImage((x2+x1+1)/2-LOGOWIDE/2, y2-LOGODEEP/2, tgeLogo);
  // lower left
  putImage(x1-LOGOWIDE/2, y2-LOGODEEP/2, tgeLogo);
  // middle left
  putImage(x1-LOGOWIDE/2, (y2+y1+1)/2-LOGODEEP/2, tgeLogo);

  setViewport(0, 0, maxx, maxy);
}


//*****
//***** Viewport and putImage() demo
//*****

void viewportPutImageDemo(int delayTime)
{
  int x, y;

  // moving putImageInv() demo (clipped)
  x = (maxx+1)/2 - LOGOWIDE/2;
  y = (maxy+1)/2 - LOGODEEP/2;

  while (!kbhit())
  {
    getImage(x, y, x+LOGOWIDE-1, y+LOGODEEP-1, bground);
    putImageInv(x, y, tgeLogo);
    if (delayTime)
      delay(delayTime);
    putImage(x>=VIEWPORTULX?x:VIEWPORTULX, y>=VIEWPORTULY?y:VIEWPORTULY,
    	bground);

    if (random(201) > 100)
      x = x<VIEWPORTLRX+LOGOWIDE/2 ? x+1 : x-1;
    else
      x = x>VIEWPORTULX-LOGOWIDE/2 ? x-1 : x+1;

    if (random(201) > 100)
      y = y<VIEWPORTLRY+LOGODEEP/2 ? y+1 : y-1;
    else
      y = y>VIEWPORTULY-LOGODEEP/2 ? y-1 : y+1;
  }

  if (getch() == ESC)
  {
    deInitGraphics();
    exit(0);
  }
}


//*****
//***** A bit of everything in a viewport
//*****

void viewportRandomDemo(void)
{
  int type;
  int x1, y1, x2, y2;

  while (!kbhit())
  {
    type = random(8);
    switch (type)
    {
      case 0:				// line()
        x1 = random(maxx+1);
        y1 = random(maxy+1);
        x2 = random(maxx+1);
        y2 = random(maxy+1);
        if (clipLine(&x1, &y1, &x2, &y2))
          line(x1, y1, x2, y2, random(maxColour));
        break;
      case 1:				// ellipse()
        ellipse(random(maxx+1), random(maxy+1), random(maxx/3),
		random(maxy/3),	random(maxColour));
        break;
      case 2:				// filledEllipse();
        filledEllipse(random(maxx+1), random(maxy+1), random(maxx/3),
		random(maxy/3), random(maxColour));
        break;
      case 3:				// circle()
        circle(random(maxx+1), random(maxy+1), random(maxx/3),
        	random(maxColour));
        break;
      case 4:				// filledCircle()
        filledCircle(random(maxx+1), random(maxy+1), random(maxx/3),
        	random(maxColour));
        break;
      case 5:				// putImageInv()
        putImageInv(random(maxx+1), random(maxy+1), tgeLogo);
        break;
      case 6:				// filledRect()
        x1 = random(maxx+1);
        y1 = random(maxy+1);
        x2 = random(maxx+1);
        y2 = random(maxy+1);
        if (clipFilledRect(&x1, &y1, &x2, &y2))
          filledRect(x1, y1, x2, y2, random(maxColour));
        break;
      case 7:
        drawRect(random(maxx+1), random(maxy+1), random(maxx+1),
		random(maxy+1), random(maxColour));
        break;
      default:
        ;
    }
  };

  if (getch() == ESC)
  {
    deInitGraphics();
    exit(0);
  }
}


//*****
//***** Palette demo
//*****

#define MAXINC	7

void paletteDemo(void)
{
  int redInc, greenInc, blueInc;
  int red=0, green=0, blue=0;
  colour = 1;

  redInc = random(MAXINC);
  redInc++;
  greenInc = random(MAXINC);
  greenInc++;
  blueInc = random(MAXINC);
  blueInc++;

  for (count=maxx; count>=0&&!kbhit(); count-=2) {
    line(0, 0, maxx-count, maxy, colour);
    line(maxx, maxy, count, 0, colour);
    cycleColour(&red, &redInc);
    cycleColour(&green, &greenInc);
    cycleColour(&blue, &blueInc);
    setPaletteReg(0, red, green, blue);
  }

  if (kbhit())
    getch();

  setPaletteReg(0, 0, 0, 0);
  clearGraphics(1);
}

void cycleColour(int *colour, int *increment)
{
  *colour += *increment;

  if (*colour < 0)
  {
    *colour = 0;
    *increment = random(MAXINC) + 1;
  }
  else if (*colour > 255)
  {
    *colour = 255;
    *increment *= -1;
  }
}


//*****
//***** Mouse demo
//*****

void mouseDemo(void)
{
  int pointerNum;

  if (!resetMouse())			// check if mouse is installed
  {
    deInitGraphics();
    unloadGraphDriver();
    printf("Microsoft or compatible mouse not detected.\n"
           "The demo for definable, device-independent pointers will not be run.\n\n"
	   "Press a key to continue...");
    getch();
    clrscr();
    signOff();
    exit(0);
  }

  initNewMouse();			// initialize new mouse handler
  setHorizLimitsMouse(0, maxx);
  setVertLimitsMouse(0, maxy);
  setPosMouse(maxx/2, maxy/2);
  pointerNum = ARROW_POINTER;
  setupMousePointer(pointerNum);
  showMouse();

  for (; !kbhit();)			// main loop
  {
    if (buttonMouse() && pointerNum!=TARGET_POINTER)
    {
      pointerNum = TARGET_POINTER;
      setupMousePointer(pointerNum);
    }
    else if (!buttonMouse() && pointerNum==TARGET_POINTER)
    {
      pointerNum = ARROW_POINTER;
      setupMousePointer(pointerNum);
    }
  }
  getch();				// flush keyboard buffer

  deInitNewMouse();			// shut off new mouse handler
}


//*****
//***** Miscellaneous routines
//*****

void forceExtension(char *filename, char *extention)
{
  char fileDrive[MAXDRIVE];
  char fileDir[MAXDIR];
  char fileFile[MAXFILE];
  char fileExt[MAXEXT];

  fnsplit(filename, fileDrive, fileDir, fileFile, fileExt);

  if (strcmpi(fileExt, extention))
    fnmerge(filename, fileDrive, fileDir, fileFile, extention);
}


void signOff(void)
{
  static char notice[] = {
    "Ŀ"
    " The Graphics Engine 1.20 Demo  Copyright (c) 1993 by Matthew Hildebrand      "
    "͵"
    " Although this demo is not particularly gripping, the power of The Graphics   "
    " Engine can be seen when it is run with different drivers:  the demo uses     "
    " the same code to access different graphics modes.                            "
    "                                                                              "
    " Even if you don't want device-independence, but only one graphics mode,      "
    " The Graphics Engine's optimized assembler routines may be used.              "
    "                                                                              "
    " The Graphics Engine costs a mere $25; paid users are entitled to free        "
    " upgrades, technical support, and the use of any or all of the code, without  "
    " royalties.  Refer to TGE.DOC for details.                                    "
    "͵"
    " Matthew Hildebrand, 4 College St., St. Catharines, Ontario, Canada, L2R 2W7  "
    ""
    "\n"
  };

  printf(notice);
}